import asyncio
import logging
from aiomeasures.util import parse_addr


class StatsDReporter:

    def __init__(self, addr, *, loop=None):
        """Sends statistics to the stats daemon over UDP

        Parameters:
            addr (str): the address in the form udp://host:port
            loop (EventLoop): the event loop
        """
        self.addr = parse_addr(addr, proto='udp')
        self.loop = loop or asyncio.get_event_loop()
        self.log = logging.getLogger(__name__)
        self.protocol = None
        self._connecting = asyncio.Lock(loop=self.loop)
        self.protocol = None

    @asyncio.coroutine
    def send(self, metrics):
        """Sends key/value pairs via UDP or TCP.
        """
        msg = bytearray()
        for i, metric in enumerate(metrics, start=1):
            msg += bytes('%s\n' % metric, encoding='utf-8')
            if i % 20 == 0:
                # small packets
                self.protocol.send(msg)
        if msg:
            self.protocol.send(msg)

    @asyncio.coroutine
    def connect(self):
        if self.protocol:
            return

        with (yield from self._connecting):
            if not self.protocol:
                transport, protocol = yield from connect(self.addr, self.loop)
                self.protocol = protocol

    def close(self):
        self.protocol.close()


class UDPProtocol(asyncio.Protocol):

    def __init__(self):
        self.log = logging.getLogger(__name__)

    def send(self, msg):
        self.log.debug('send %s', msg)
        self.transport.sendto(msg)

    def connection_made(self, transport):
        addr = '%s:%s' % transport.get_extra_info('peername')
        self.log.info('connected to %s', addr)
        self.transport = transport

    def datagram_received(self, data, addr):
        self.log.debug('received %s', data.decode())

    def error_received(self, exc):
        addr = '%s:%s' % self.transport.get_extra_info('peername')
        self.log.warning('error received %s %s', addr, exc)

    def connection_lost(self, exc):
        self.log.warning("socket closed")

    def close(self):
        if self.transport:
            self.transport.close()


@asyncio.coroutine
def connect(addr, loop):
    if addr.proto == 'udp':
        transport, protocol = yield from loop.create_datagram_endpoint(
            lambda: UDPProtocol(),
            remote_addr=addr
        )
        protocol.transport = transport
    else:
        raise NotImplementedError()
    return transport, protocol



__all__ = ['Check']


class Check:

    __slots__ = ('name', 'status', 'timestamp', 'hostname', 'tags', 'message')

    def __init__(self, name, status, timestamp=None,
                 hostname=None, tags=None, message=None):
        self.name = name
        self.status = status
        self.timestamp = timestamp
        self.hostname = hostname
        self.tags = tags
        self.message = message

    def __repr__(self):
        args = ['%s=%s' for attr in ()]
        attrs = ('name', 'status', 'tags')
        for attr in attrs:
            value = getattr(self, attr, None)
            if value is not None:
                args.append('%s=%r' % (attr, value))
        return '<%s(%s)>' % (self.__class__.__name__, ', '.join(args))



"""
    AIO Measures
    ~~~~~~~~~~~~

"""

from ._version import get_versions
from .checks import *
from .clients import *
from .events import *
from .metrics import *

__all__ = (checks.__all__
           + clients.__all__
           + events.__all__
           + metrics.__all__)

__version__ = get_versions()['version']
del get_versions



#!/usr/bin/env python
from setuptools import setup, find_packages, Command
import versioneer


class PyTest(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import subprocess
        import sys
        errno = subprocess.call([sys.executable,
                                 'runtests.py',
                                 'tests/',
                                 '--cov', 'aiomeasures',
                                 '--cov-report', 'html'])
        raise SystemExit(errno)

cmds = versioneer.get_cmdclass()
cmds.update({'test': PyTest})

setup(
    name='aiomeasures-fork',
    version=versioneer.get_version(),
    description="Collect and send metrics to StatsD",
    author="Xavier Barbosa",
    author_email='clint.northwood@gmail.com',
    url='http://lab.errorist.xyz/abc/',
    packages=find_packages(),
    keywords=[''],
    install_requires=[],
    extras_require={
        ':python_version=="3.3"': ['asyncio', 'singledispatch'],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    license='MIT',
    cmdclass=cmds
)



import asyncio
from abc import ABCMeta, abstractmethod
from aiomeasures.checks import Check
from aiomeasures.events import Event
from aiomeasures.metrics import CountingMetric, GaugeMetric
from aiomeasures.metrics import HistogramMetric, SetMetric, TimingMetric
from time import perf_counter


class Client(metaclass=ABCMeta):

    def incr(self, name, value=None, rate=None, tags=None):
        value = abs(value or 1)
        metric = CountingMetric(name, value, rate=rate, tags=tags)
        return self.register(metric)

    def decr(self, name, value=None, rate=None, tags=None):
        value = -abs(value or 1)
        metric = CountingMetric(name, value, rate=rate, tags=tags)
        return self.register(metric)

    def counter(self, name, value, rate=None, tags=None):
        metric = CountingMetric(name, value, rate=rate, tags=tags)
        return self.register(metric)

    def timing(self, name, value=None, rate=None, tags=None):
        metric = TimingMetric(name, value, rate=rate, tags=tags)
        return self.register(metric)

    def timer(self, name, rate=None, tags=None):
        return Timer(client=self, name=name, rate=rate, tags=tags)

    def gauge(self, name, value, rate=None, delta=False):
        metric = GaugeMetric(name, value, rate=rate, delta=delta)
        return self.register(metric)

    def histogram(self, name, value, rate=None, delta=False):
        metric = HistogramMetric(name, value, rate=rate, delta=delta)
        return self.register(metric)

    def set(self, name, value, rate=None, tags=None):
        metric = SetMetric(name, value, rate=rate, tags=tags)
        return self.register(metric)

    def event(self, title, text, **kwargs):
        event = Event(title, text, **kwargs)
        return self.register(event)

    def check(self, name, status, **kwargs):
        check = Check(name, status, **kwargs)
        return self.register(check)

    @abstractmethod
    def format(self, metric, prefix=None):
        raise NotImplementedError()

    @asyncio.coroutine
    @abstractmethod
    def send(self):
        """Sends key/value pairs via UDP or TCP.
        """
        raise NotImplementedError()

    @abstractmethod
    def register(self, metric):
        raise NotImplementedError()

    @abstractmethod
    def close(self):
        raise NotImplementedError()


class Timer:

    def __init__(self, client, name, rate=None, tags=None):
        self.client = client
        self.name = name
        self.rate = rate
        self.tags = tags

    def __call__(self, func):
        def wrapper(*args, **kwargs):
            try:
                self.start()
                return func(*args, **kwargs)
            finally:
                self.stop()
        return wrapper

    def __enter__(self):
        self.start()

    def __exit__(self, type, value, tb):
        self.stop()

    def start(self):
        self._started = perf_counter()

    def stop(self):
        value = int((perf_counter() - self._started) * 1000)
        self.client.timing(self.name, value, rate=self.rate, tags=self.tags)
